// * 2- and 3-Body Demo

comment
Two and three body problems.
endcomment

function f2body (x,y,m1,m2)
## y=f(x,y) for the differential equations
	y1=y[1:2]; y2=y[3:4];
	d21=(y2-y1)/norm(y2-y1)^3;
	return y[5:8]|d21*m2|-d21*m1;
endfunction

function test2body (y1,m1=1,m2=1,n=100,tmax=360,fixed=0,h=1)
## 2-body demo
##
## Intial y is a row vector with two positions and two speed vectors
## in the form [x1,y1,x2,y2,vx1,vy1,vx2,vy2]. m1 and m2 are the masses
## of the two bodies. 
##
## For slower animation increase h. For more accuracy increase n. The
## simulation runs until one body leaves the definition area (unless
## fixed is true), or tmax is reached.
##
## >fixed fixes the display at the first mass, which is positioned
## in (0,0).
	y=y1;
	x=0;
	setplot(-3,3,-3,3); clg; hold off; xplot(); frame();
	if fixed then 
		hold on; color(12); markerstyle("ow");
		mark(0,0); hold off; 
	endif;
	setstatus("Any key stops the movement");
	repeat;
		ynew=adaptiverunge("f2body", ..
			linspace(x,x+h,n),y,0.0001,h;m1,m2);
		H=ynew;
		hold on;
		if fixed then
			color(13); plot(H[3]-H[1],H[4]-H[2]);
		else
			color(12); plot(H[1],H[2]);
			color(13); plot(H[3],H[4]);
		endif;
		hold off;
		wait(0);
		y=ynew[,-1]'; x=x+h;
		if (any(abs(y[1:4])>4));
			"One body left definition area!",
			break;
		endif;
		wait(h/10);
		until testkey;
		until x>tmax;
	end;
	return x
endfunction

// * Functions

function f3body (x,y,m1,m2,m3)
## y=f(x,y) for the differential equations
	y1=y[1:2]; y2=y[3:4]; y3=y[5:6];
	d21=(y2-y1)/norm(y2-y1)^3*m1*m2;
	d31=(y3-y1)/norm(y3-y1)^3*m3*m1;
	d32=(y3-y2)/norm(y3-y2)^3*m3*m2;
	return y[7:12]|(d21+d31)/m1|(-d21+d32)/m2|(-d31-d32)/m3;
endfunction

function test3body (y1,m1=1,m2=1,m3=1,n=100,fixed=0,tmax=360,h=1)
## 3-body demo
##
## Works like test2body, but takes three positions and speed vectors,
## and three masses.
	y=y1;
	x=0;
	setplot(-3,3,-3,3); clg; hold off; xplot(); frame();
	if fixed then 
		hold on; color(12); markerstyle("ow");
		mark(0,0); hold off; 
	endif;
	setstatus("Any key stops the movement");
	repeat;
		ynew=adaptiverunge("f3body", ..
			linspace(x,x+h,n),y,0.0001,h;m1,m2,m3);
		H=ynew;
		hold on;
		if fixed
			color(13); plot(H[3]-H[1],H[4]-H[2]);
			color(14); plot(H[5]-H[1],H[6]-H[2]);
		else
			color(12); plot(H[1],H[2]);
			color(13); plot(H[3],H[4]);
			color(14); plot(H[5],H[6]);
		endif;
		hold off;
		wait(0);
		y=ynew[:,-1]'; x=x+h;
		if !fixed and (any(abs(y[1:6])>4)) then
			"One body left definition area!",
			break;
		endif;
		wait(h/10);
		until testkey;
		until x>tmax;
	end;
	return x
endfunction

// * Set Planets

function setplanets
## Set the planets and initial vectors with the mouse.
##
## 3-body which sets the positions with the mouse.
	y=zeros(1,12);
	setplot(-3,3,-3,3); hold off; clg; xplot(); frame();
	setstatus("Set the places and vectors for all three bodies");
	loop 1 to 3;
		m=mouse; style("m[]");
		hold on; mark(m[1],m[2]); hold off;
		y[2*#-1:2*#]=m;
		m1=mouse;
		style("->"); color(3+#);
		hold on; plot([m[1],m1[1]],[m[2],m1[2]]); hold off;
		color(1); style("");
		y[6+2*#-1:6+2*#]=(m1-m)/10;
	end;
	return test3body(y);
endfunction


