#include "dlldef.h"
#include <string.h>

#define BYTE unsigned char

void makeprimes (BYTE *b, int k)
// make primes using the sieve of Eratosthenes
{
	int i,j,step;
	for (i=0; i<k; i++) b[i]=1;
	for (i=0; i<k; i++)
	{
		if (b[i]==1)
		{
			step=2*i+3; // b contains the odd numbers
			for (j=i+step; j<k; j+=step) b[j]=0;
		}
		if ((2*i+1)*(2*i+1)>2*k+1) break;
	}
}

EXPORT char * dllprimes (header * p[], int np,
	char *rstart, char *rend)
// Get and check the Euler parameters, run algorithm, and return result.
// See the HTML documentation for more information.
{
	start(rstart,rend); // necessary start routine
	
	double x;
	x=getreal(p[0]); // get the real number and check param type
	IFERROR("Need a real for dllprimes.");
	
	int n=(int)x; // convert to integer
	if (n!=x || n<=2) ERROR("Need positive integer larger than 2");
	
	// get byte array from stack
	int k=(n-3)/2+1;	
	BYTE *b=(BYTE *)getram(k*sizeof(BYTE));
	CHECK(b,"Out of heap space.");
	
	makeprimes(b,k); // run the sieve
	
	// count prime numbers
	int i,count=0; 
	for (i=0; i<k; i++) if (b[i]) count++;
	
	// make output vector on stack
	header *out;
	out=new_matrix(1,count+1);
	IFERROR("Out of stack space in dllprimes.");
	double *m=matrixof(out);
	m[0]=2;
	
	// copy primes into it
	count=1;
	for (i=0; i<k; i++) if (b[i]) { m[count]=(2*i+3); count++; }
	
	moveresults(out); // necessary moving of result
	
	return newram; // always return newram, ram or 0
}
