// * Demo of the Nelder Mead method

// The code here is simiolar to the code of the built-in Nelder Mead
// function.

function nelderstep (s,fff)
## One Nelder-Mean step from the polyto s
##
## This minimizes the function fff (one step of the Nelder Mead
## algorithm).
##
## s should be (n+1)xn
## fff can evaluate at 1xn vectors
	n=cols(s);
	f=zeros(1,n+1);
	loop 1 to n+1;
		f[#]=fff(s[#]);
	end
	{f,ind}=sort(f); s=s[ind];
	z=(sum(s[1:n,:]')/n)';
	d=z-s[n+1];
	sr=z+d; fr=fff(sr);
	shrink=0;
	if fr<f[1] then // try to extend the simplex
		se=z+2*d; fe=fff(se);
		if fe<fr; s[n+1]=se; else; s[n+1]=sr; endif;
	elseif fr<f[n] then s[n+1]=sr;
	elseif fr<f[n+1] then
		so=z+d/2; fo=fff(so);
		if fo<=f[n+1]; s[n+1]=so; 
		else shrink=1;
		endif;
	else
		si=z-d/2; fi=fff(si);
		if fi<=f[n+1] then s[n+1]=si;
		else shrink=1;
		endif;
	endif;
	if shrink then
		loop 2 to n+1;
			s[#]=(s[1]+s[#])/2;
		end;
	endif;
	return s;
endfunction

function nelderdemo (s,fff)
## Demo of the Nelder Mead method in 2 variables
##
## Plots each step.
##
## Returns the last simplex s
	hold on; 
	c=color(red);
	title("Press scpace or return!");
	h=(s_s[1])'; plot(h[1],h[2]);
	repeat
		if key("Press return")!=32 then 
			hold off; color(c); break; 
		endif;
		s=nelderstep(s,fff);
		h=(s_s[1])'; plot(h[1],h[2]);
	end
	return s;
endfunction
