function rk4 (ffunction,t,y0)
 ## y=rk4("f",t,y0,...) solves the differential equation y'=f(t,y).
 ## f(t,y,...) must be a function.
 ## y0 is the starting value.
        useglobal; 
 	n=cols(t);
 	y=dup(y0,n);
 	loop 1 to n-1;
 	h=t[#+1]-t[#];
 	yh=y[#]; xh=t[#];
 	k1=ffunction(xh,yh,args(4));
 	k2=ffunction(xh+h/2,yh+h/2*k1,args(4));
        k3=ffunction(xh+h/2,yh+h/2*k2,args(4));
 	k4=ffunction(xh+h,yh+h*k3,args(4));
 	y[#+1]=yh+h/6*(k1+2*k2+2*k3+k4);
 	end;
 	return y';
 endfunction
 function rhs(t, T, k, Ts)
 ## calculates right hand side of equation dT/dt = -k*(T-Ts)
          useglobal;      
          return -k*(T-Ts)
 endfunction
 
 function fun1(k, Ts, T0, t0, t1, h)
 ## calculates and plots solution of equation dT/dt =- k*(T-Ts)
     useglobal;
     t = t0:h:t1;
     T = rk4("rhs", t, T0, k, Ts);
     shrinkwindow();
     setplot(0, 50, 50, 350);
     plot(t, T);
     xgrid(0:10:50,1); ygrid(50:50:350,1);
     title("Pie Temperature");
     return T;
 endfunction
 
 function fun2(k, Ts, T0, T1, t0, t1, n)
 ## calculates T(t1)-T1
     useglobal;
     h = (t1-t0)/n;
     t = t0:h:t1;
     T = rk4("rhs", t, T0, k, Ts);
     return T[length(t)]-T1;
 endfunction
 
 function fun3(t2, k, Ts, T0, T2, t0, n)
 ## calculates T(t2)-T2 
     useglobal;
     h = (t2-t0)/n;
     t = t0:h:t2;
     T = rk4("rhs", t, T0, k, Ts);
     return T[length(t)]-T2;
 endfunction
 
 function fun4(k, Ts, T0, t0, t1, n)
 ## calculates T(t1)
     useglobal;
     h = (t1-t0)/n;
     t = t0:h:t1;
     T = rk4("rhs", t, T0, k, Ts);
     return T[length(t)];
 endfunction
