function nlnfit(x,y,b,ffunction,d=0.1,eps=epsilon())
##Returns a vector representing the fitting parameters b
##in ffunction(x,b)
## x - vector (matrix) of independent variable(s)
## y - vector of dependent variable (the same length as x)
## b - vector of initial guess for unknown parameters
## ffunction(x,b) - returns a vector of fitting function
## d - initial step size (see brentmin,neldermin)
## eps - the final accuracy
##
## Example:
## given: vectors x,y,
## guess: vestor b
## function: y = 10^(b_1 + b_2/x + b_3*log10(x) + b_4*x^2)
## ..
## function ffunction(x,b)
##  return 10^(b_1 + b_2/x + b_3*log10(x) + b_4*x^2);
## endfunction;
##..
## nlnfit returns vector: b_1,b_2,b_3,b_4
printf("initial sum of squared resid.=%15.5f",ffsumsq(b,x,y,ffunction))
if cols(b)==1;
       bfit=brentmin("ffsumsq",b,d,eps;x,y,ffunction);
else;
       bfit=neldermin("ffsumsq",b,d,eps;x,y,ffunction);
endif;
printf("final sum of squared resid.=%15.5f",ffsumsq(bfit,x,y,ffunction))
return bfit;
endfunction;

function ffsumsq(b,x,y,ffunction)
## calculates sum of squared residuals
## between vector of dependent variable and
## valuse obtained by fitting function
## (needed for nlnfit)
suma = sum((y-ffunction(x,b))^2);
return suma
endfunction;

