comment
Wegstein method
endcomment
function wegstein (ffunction,x0,w=0,n=0)
## wegstein("f",x0;...) iterates the function f(x,...), starting from
## x0 using wegstein method
## w - integer (0,1,2...), w=0 - complete wegstein iterations,
## (use nwegstein in order to choose w)
## The iteration stops at a fixed point.
## Returns the fixed point.
## ## You can specify an epsilon eps with eps=... as last parameter.
        if n>0;
		return nwegstein(ffunction,x0,w,n;args());
	endif;
	if (isvar("eps")); localepsilon(eps); endif;
        m=cols(x0);
	a=zeros(1,m);q=zeros(1,m);
        x=zeros(1,m);x1=zeros(1,m);x2=zeros(1,m); 
        x1=x0; 
        if isfunction(ffunction);
          x2=ffunction(x,args());
        else;
          x2=expreval(ffunction,x);
        endif;
        iter=0;      
  	repeat;
         if iter==w;
                 iter=0; 
                 if isfunction(ffunction);
                    f1=ffunction(x1,args());
                    f2=ffunction(x2,args());
                 else;
                    f1=expreval(ffunction,x1);
                    f2=expreval(ffunction,x2);
                 endif;
                 loop 1 to m;
                  a=f2-f1; 
                  if (x2[#]~=x1[#]);
                      a=a*abs(x2[#]-x1[#])/epsilon;
                  else;
                      a=(f2[#]-f1[#])/(x2[#]-x1[#]);
                  endif;  
                      q[#]=a/(a-1);
                 end;     
                 q=(q<=-5 || q>=0)*q;
                 x=q*x2+(1-q)*f2;
                 if (x~=x2);return x;endif
         else;
                 iter=iter+1;
                 if (x1~=x2);return x2;endif
                 if isfunction(ffunction);
                    x=ffunction(x2,args());
                 else;
                    x=expreval(ffunction,x2);
                 endif;                
         endif;
		 x1=x2;x2=x;  
	end;     	
endfunction

function nwegstein (ffunction,x0,w,n)
## nwegstein("f",x0,w,n;...) Iterate the function f(x,...) n times,
## starting with the point x0 using wegstein method.
## w - integer (0,1,2...), w=0 - complete wegstein iterations,
## start w=0, inrease by 1 until satisfactory convergence acheived  
## Returns the vector of iterants.
        m=cols(x0);
	y=zeros(n,m);a=zeros(1,m);q=zeros(1,m);
        x=zeros(1,m);x1=zeros(1,m);x2=zeros(1,m); 
        x1=x0; 
        if isfunction(ffunction);
          x2=ffunction(x1,args());
        else;
          x2=expreval(ffunction,x1);
        endif;
        iter=0;      
  	loop 1 to n;
         if iter==w;
                 iter=0; 
                 if isfunction(ffunction);
                    f1=ffunction(x1,args());
                    f2=ffunction(x2,args());
                 else;
                    f1=expreval(ffunction,x1);
                    f2=expreval(ffunction,x2);
                 endif;
                 loop 1 to m;
                  a=f2-f1; 
                  if (x2[#]~=x1[#]);
                      a=a*abs(x2[#]-x1[#])/epsilon;
                  else;
                      a=(f2[#]-f1[#])/(x2[#]-x1[#]);
                  endif;
                      q[#]=a/(a-1);
                 end;     
                 q=(q<=-5 || q>=0)*q;
                 x=q*x2+(1-q)*f2;
         else;
                 iter=iter+1;
                 if isfunction(ffunction);
                    x=ffunction(x2,args());
                 else;
                    x=expreval(ffunction,x2);
                 endif;
         endif;
		 y[#,:]=x;
                 x1=x2;x2=x;  
	end;     
	return y;
endfunction
