// Affine Scaling

// This a method due to Vanderbei e.a. in the 1980s.

function affinescaling (A:real, b:real column, c: real vector,
	gamma:positive number=0.9, x:real column=none,
	history=false, infinite=1/epsilon)
## Minimize c.x with A.x=b using an inner method
##
## The method used it due to Vanderbei e.a. It takes the current
## solution x to (1,...,1) and uses a direction c projected to the
## kernel of A.
##
## The start vector x must be an inner vector. If it is missing, an
## additional variable is used, and A is expanded such that
## x=(1,...,1) is a feasible start point.
##
## A : mxn matrix
## b : 1xm column vector
## c : nx1 row vector
## gamma : 0<gamma<1 (optional)
## x : 1xn start vector (optional)
##
## Returns : xopt (1xn) vector
##
## history : If true {xopt,X} is returned, where X contains all points
## visited.
	n=cols(A);
    if x==none then
    	u=b-sum(A);
    	A=A|u;
    	x=ones(cols(A),1);
    	c=c|infinite;
    endif;
	z=c.x;
	if history then X=x; endif;
	repeat
		d=ones(1,cols(A));
		d=x';
		tA=A*d; tc=c*d;
		y=fit(tA',tc');
		v=tc'-tA'.y;
		while !all(v~=0);
		if all(v<=0) then
			error("Problem unbounded");
		endif;
		i=nonzeros(v>0);
		lambda=gamma/max(v[i]');
		tx=x/d'-lambda*v;
		x=d'*tx;
		znew=c.x;
		if history then X=X|x; endif;
		while znew<z;
		z=znew;
	end
	if history then return {x[1:n],X[1:n]};
	else return x[1:n];
	endif;
endfunction
