function next (delay=5)
	{d,c}=wait(delay);
	if d<delay && c==13 then return 1; endif;
	return 0;
endfunction

function demo ()
	repeat;
	reset();
	
	// x^x and tangents
	expr="x^x";
	plot2d(expr,a=0,b=1,c=0.5,d=1,title="x^x");
	x=[0.2,1/E,0.6];
	loop 1 to length(x)
		t=x[#]; y=expr(t); dy=diff(expr,t); 
		plot2d(t,y,points=1,add=1);
		plot2d([t-0.1,t+0.1],[y-0.1*dy,y+0.1*dy],color=2,add=1);
		wait(0.5);
	end;
	label("x=1/E",1/E,expr(1/E));
	if next() then break; endif;
	
	// sinc and Riemann sum
	expr="sinc(x)";
	x=linspace(0,pi,20);
	y=expr(tail(x,2));
	plot2d(x,y,a=0,b=pi,c=-0.1,d=1.1,bar=1,style="/");
	wait(0.5);
	plot2d(expr,a=epsilon,b=pi,add=1,title="sin(x)/x");
	wait(0.5);
	text("Integral "|integrate(expr,0,pi),700,120);
	text("Sum      "|sum(y)*(x[2]-x[1]),700,150);
	if next() then break; endif;
	
	// Chebyshev polynomials
	plot2d(none,r=1);
	loop 1 to 6;
		plot2d("cheb",-1,1;#,color=rgb(0,#*0.1,1-#*0.1),..
			add=1,title="Chebyshev polynomials");
	wait(0.5);
	end
	if next() then break; endif;

	// variety of functions
	v=10; g=10;
	t=0:0.1:2;
	a=(5:5:85); a=a';
	plot2d(cos(a)*v*t,sin(a)*v*t-g*t^2/2, ..
		a=0,b=10,c=0,d=10,title="Ballistic shots");
	d=interp([-10,0,10],[0,5,0]);
	wait(0.5);
	plot2d("interpval([-10,0,10],d,x)";d,add=1,color=10,thickness=3);
	if next() then break; endif;
	
	// x^y
	x=0:0.005:1; y=0.1:0.1:1;
	plot3d(x,y',x^y',angle=160,title="x^y",lines=1,n=20);
	if next() then break; endif;
	
	// random points
	loop 1 to 5
		plot2d(normal(1,500),normal(1,500),points=1,style="..",..
			title="500 Random points");
		wait(1);
	end;
	if next() then break; endif;
	
	// random walks
	figure(2,2);
	title("Random walks");
	loop 1 to 4;
 		figure(#);
		plot2d(1:500,cumsum(normal(1,500)));
		wait(0.5);
	end;
	if next() then break; endif;
	figure(0);
	
	// random distribution
	v=normal(1,100000);
	plot2d(v,distribution=1,title="Distribution of 100'000 points",..
		style="\/");
	plot2d("qnormal",add=1,thickness=2);
	if next() then break; endif;
	
	// web plot
	fwebplot("cos(x)",0,1,0.5,20);
	title("Web plot of x[n+1]=cos(x[n])");
	if next() then break; endif;	

	// Simplex	
	phi=(0:6)/7*2pi;
	A=(cos(phi)_sin(phi))';
	b=3*ones(rows(A),1)+5*sum(A);
	x=feasibleArea(A,b);
	plot2d(x[1],x[2],polygon=1,filled=1,style="\",a=0,b=10,c=0,d=10);
	x=simplex(A,b,[1,1],max=1); plot2d(x[1],x[2],points=1,add=1);
	plot2d([x[1]-1,x[1]+1],[x[2]+1,x[2]-1],thickness=2,color=5,add=1);
	title("Linear Optimization");
	if next() then break; endif;	
	
	// Integer Simplex
	phi=(0:6)/7*2pi;
	A=(cos(phi)_sin(phi))';
	b=3*ones(rows(A),1)+5*sum(A);
	x=feasibleArea(A,b);
	plot2d(x[1],x[2],polygon=1,filled=1,style="\",a=0,b=10,c=0,d=10);
	x=intsimplex(A,b,[1.1,1],max=1); plot2d(x[1],x[2],points=1,add=1);
	plot2d([x[1]-1,x[1]+1],[x[2]+1,x[2]-1],thickness=2,color=5,add=1);
	title("Integer Linear Optimization");
	if next() then break; endif;
	
	// Euler spiral
	t=linspace(0,4pi,800);
	x=heun("sin(x^2/2)",t,0);
	y=heun("cos(x^2/2)",t,0);
	setplot(-1.4,1.4,-1.4,1.4);
	clg; hold; plot(flipx(-x)|x,flipx(-y)|y); hold;
	title("Euler spiral");
	reset();
	if next() then break; endif;

	// implicit plots
	plot2d("x^y-y^x",a=0,b=5,c=0,d=5,niveau=0, ..
		title="Solutions of x^y=y^x");
	label("(E,E)",E,E);
	if next() then break; endif;
	
	// contour plot
	plot3d("x^y-y^x",a=0,b=5,c=0,d=5,niveau=0, ..
		title="Solutions of x^y=y^x");
	if next() then break; endif;

	// wave in polar plot
	plot3d("cos(sqrt(x^2+y^2))*exp(-sqrt(x^2+y^2)/12)", ..
		  r=8pi,hue=1,fscale=1,polar=1,n=200,frame=0,scale=1.8, ..
		  title="Wave");
	if next() then break; endif;

	// Moebius strip
	u=linspace(-1,1,10);
	v=linspace(0,2*pi,50)';
	X=(3+u*cos(v/2))*cos(v); Y=(3+u*cos(v/2))*sin(v); Z=u*sin(v/2);
	plot3d(X,Y,Z,n=20,frame=0,scale=1.5,title="Moebius strip");
	if next() then break; endif;

	// Taylor series
	for i=0 to 10 step 2
		plot2d("cos(x)",a=-2*pi,b=2*pi,c=-3,d=3, ..
			title="Taylor series to sin(x)");
		df="::taylor(cos(x),x,0,@i)";
		plot2d(df,add=1,color=3);
		text("sin(x) ~= "|df,100,100);
		wait(0.5);
	end;
	if next() then break; endif;
	
	// random matrix 
	A=cumsum(random(10,10));  
	plot3d(A,bar=1,angle=270,xlabel="",ylabel="",title="Random Matrix");
	if next() then break; endif;
	
	// contour lines on ellipse
	n=100; phi=linspace(-pi,pi,n); psi=linspace(-pi/2,pi/2,n/2)';
	x=cos(phi)*cos(psi); y=sin(phi)*cos(psi); z=sin(psi)*1.5;
	v=x+y-z;
	plot3d(x,y,z,hue=1,niveau=-3:0.2:3,values=v, ..
		title="Ellipsoid cut by parallel planes");
	if next() then break; endif;
	
	// Implicit plot
	plot3d("((x-1)^2+y^2+z^2)^2*((x+1)^2+y^2+z^2)^2-1.2", ...
		  implicit=3,r=1.5,frame=0,scale=1.5, ..
		  title="((x-1)^2+y^2+z^2)^2*((x+1)^2+y^2+z^2)^2=1.2");
	if next() then break; endif;
	
	// Axis plot
	f3daxispolar("x^2+y^2");
	title("x^2+y^2");
	if next() then break; endif;
	
	// Rotation
	n=120; d=0.01;
	deletepages();
	clg;
	title("Creating Animation");
	addpage();
	showpage(1);
	t=linspace(0,2*pi,n);
	v=view();
	loop 1 to cols(t);
		  plot3d("x^2+y^3",title="x^2+y^3",angle=t[#],frame=0);
		  addpage();
	end;
	loop 1 to 2
	loop 2 to pages();
	showpage(#);
	end;
	end;
	showpage(0);
	deletepages();
	reset();
	if next() then break; endif;
	
	// Spline
	x=random(1,10)-0.5; y=random(1,10)-0.5;
	p=1:10; sx=spline(p,x); sy=spline(p,y);
	t=linspace(1,10,500);
	plot2d(splineval(p,x,sx,t),splineval(p,y,sy,t),r=0.7);
	plot2d(x,y,points=1,style="o",add=1,color=2);
	title("Natural spline through random points");
	if next() then break; endif;	
	
	// sliced rotation
	x=(0:0.05:2)'; r=exp(-x);
	phi=linspace(0,2*pi,50);
	y=cos(phi)*r; z=sin(phi)*r;
	plot3d(x,y,z,disconnect=2:2:length(x));
	title("exp(-x) rotated");
	if next() then break; endif;
	
	// Astroide
	a=(0.05:0.05:1)';
	plot2d("-(a*x-a*sqrt(1-a^2))/sqrt(1-a^2)";a, ..
		a=0,b=1,c=0,d=1);
	plot2d("sin(x)^3","cos(x)^3", ..
		xmin=0,xmax=pi/2,color=2,thickness=2,add=1, ..
		title="Astroide");
	if next() then break; endif;
	
	end; // eternal loop

endfunction

function computeLorenz (%M,pts,n,s,b,r,dt)
    for i=1 to n;
	    x=%M[1];
	    y=%M[2];
	    z=%M[3];
	    pts[1,i]=(s*(y-x))*dt+x;
	    pts[2,i]=(r*x-y-x*z)*dt+y;
	    pts[3,i]=(x*y-b*z)*dt+z;
	    %M[1]=pts[1,i];
	    %M[2]=pts[2,i];
	    %M[3]=pts[3,i];
	end;
    return 0;
endfunction

function ademo
	repeat
	
	// simple wire plot
	plot3d("x*y",wire=1,anaglyph=1,angle=-70,title="x*y");
	if next() then break; endif;
	
	// simple contour plot
	plot3d("x*y^3",contour=1,anaglyph=1,title="x*y^3")
	if next() then break; endif;
	
	// contours only
	plot3d("x^2+y^3",r=2,anaglyph=1,contour=2,zoom=2,..
		title="x^2+y^3");
	if next() then break; endif;

	// implicit plot
	plot3d("x^2*y-y^3-z^2",implicit=5,r=2,angle=70,height=30, ..
		anaglyph=1,maps=0,title="x^2*y-y^3-z^2");
	if next() then break; endif;

	// Moebius strip
	x=linspace(0,2*pi,100); y=(-1:0.1:1)';
	plot3d(cos(x)*(1+y/2*cos(x/2)),sin(x)*(1+y/2*cos(x/2)),y/2*sin(x/2), ..
		frame=0,wire=1,anaglyph=1,max=0.8,scale=1.5, ..
		title="Moebius strip");
	if next() then break; endif;

	// Random Walk
	A=cumsum(normal(3,1000)); 
	plot3d(A[1],A[2],A[3],wire=1, ..
		anaglyph=1,title="Random Walk");
	if next() then break; endif;
	
	// Cloud of Points
	A=normal(3,1000);
	plot3d(A[1],A[2],A[3],points=1,style=".", ..
		anaglyph=1,title="1000 random points");
	if next() then break; endif;

	// Lorenz attractor
	n=10000;
	pts=ones(3,n);
	s=10;
	b=8/3;
	r=27;
	dt=0.01;
	M=normal(1,3)/1000000;
	computeLorenz(M,pts,n,s,b,r,dt);
	plot3d(pts[1],pts[2],pts[3], ..
		anaglyph=1,wire=1,anaglyph=1,frame=0);
	if next() then break; endif;
	
	// knot
	u=linspace(-%pi,%pi,80); v:=linspace(-%pi,%pi,200)';
	x=(4*(1+.25*sin(3*v))+cos(u))*cos(2*v);
	y=(4*(1+.25*sin(3*v))+cos(u))*sin(2*v); z=sin(u)+2*cos(3*v);
	plot3d(x,y,z,frame=0,scale=1.5, ..
		anaglyph=1,hue=1,light=[1,0,-1]); 
	if next() then break; endif;	

	end; // eternal loop

endfunction
