comment
Two problems:
- Child and toy problem.
- Man and dog problem.
Load the child notebook for a demonstration.
endcomment

function childsin (t)
## Example for a path of the child.
## The function must return the path and its derivative.
## Child moves along a circle.
	return {[t,sin(t)],[1,cos(t)]}
endfunction

function childcircle (t)
## Example for a path of the child.
## Child moves along a circle
	return {[cos(t),sin(t)],[-sin(t),cos(t)]}
endfunction

function fchild (t:scalar,x:vector,f:string)
## Defines f(t,x) for the differential equation.
## f is one of the example functions, returning
## the path at t and the derivative vector at t.
	{xc,vc}=f(t);
	w=x-xc;
	w=w/sqrt(sum(w*w));
	return (vc.w')*w;
endfunction

function child (f:string,start=[2,0],tend=10,n=200, ..
	a=-2,b=2,c=-2,d=2,r=0)
## A child walks along a curve described by f from time 0 to tend.
## It holds a toy on a stick, which is initially at start.
## The function displays the paths of the child and the toy.
## Returns {child,toy} : child and toy paths.
	t=linspace(0,tend,n);
	// compute child path:
	child=zeros(n+1,2);
	loop 1 to n+1;
		child[#]=f(t[#];args());
	end;
	child=child';
	// compute toy path:
	toy=heun("fchild",t,start;f,args());
	// plot:
	if r>0 then a=-r; b=r; c=-r; d=r; endif;
	plot2d(child[1],child[2],a=a,b=b,c=c,d=d,thickness=2,color=5);
	plot2d(toy[1],toy[2],add=1,thickness=2,color=6);
	// return both paths
	return {child,toy};
endfunction

function dogcircle (t)
## Example for a path for the dog problem.
## Must return the path of the man.
	return [cos(t),sin(t)];
endfunction

function dogline (t)
## Example for a path for the dog problem.
	return [t,0];
endfunction

function fdog (t:scalar,x:vector,f:string,speed:scalar)
## Defines the differential equation y'=fdog(t,x).
	xc=f(t);
	w=xc-x;
	w=w/sqrt(sum(w*w));
	return speed*w;
endfunction

function dog (f:string,start=[2,0],tend=10,speed=1, ..
	n=200,a=-2,b=2,c=-2,d=2,r=0)
## A child walks along a curve described by f from time 0 to tend.
## A dog runs towards the child with constant speed.
## The function displays the paths of the child and the dog.
## Returns {child,dog} : paths of child and toy
	t=linspace(0,tend,n);
	child=zeros(n+1,2);
	loop 1 to n+1;
		child[#]=f(t[#];args());
	end;
	child=child';
	dog=heun("fdog",t,start;f,speed,args());
	if r>0 then a=-r; b=r; c=-r; d=r; endif;
	plot2d(child[1],child[2],a=a,b=b,c=c,d=d,thickness=2,color=5);
	plot2d(dog[1],dog[2],add=1,thickness=2,color=6);
	return {child,dog}
endfunction

function animateresult (child,toy)
## Animate the solutions of the above problems
	deletepages();
	clg; title("Creating Animation"); addpage(); showpage(1);
	loop 1 to cols(child)
		plot2d(child[1],child[2],r=2,color=5,thickness=2);
		plot2d(toy[1],toy[2],color=6,add=1,thickness=2);
		plot2d(child[1,#],child[2,#],color=5,points=1,add=1,style="o");
		plot2d(toy[1,#],toy[2,#],color=6,points=1,add=1,style="o");
		plot2d([child[1,#],toy[1,#]],[child[2,#],toy[2,#]],add=1,thickness=3);
		addpage();
		end;
	animate();
	deletepages();
endfunction
