function %fv (t, f, x, v) := f(x+t*v);

function vmin (f:string, x:real vector, v:real vector, 
	tstart:number, tmax:number=1, tmin:number=epsilon,
	overshoot:real=1)
## Minimize f(x+t*v)
	y=f(x);
	t=tstart;
	repeat
		ynew=f(x+t*v);
		while ynew<y;
		t=t*2;
		until t>tmax;
	end;
	t=fmin("%fv",0,t;f,x,v);
	return x+t*v;
endfunction

function fgradientmin (f:string, df:string, x:real vector,
	tstart:number, tmax:number=1, tmin:number=epsilon,
	overshoot:real=1)
## Minimize f(x) using the gradient for descent.
endfunction

function nelderstep (s,fff)
## Do one Nelder-Mean step from the polypon s
## minimizing the function fff.
## s should be (n+1)xn
## fff can evaluate at 1xn vectors
	n=cols(s);
	f=zeros(1,n+1);
	loop 1 to n+1;
		f[#]=fff(s[#]);
	end
	{f,ind}=sort(f); s=s[ind];
	z=(sum(s[1:n,:]')/n)';
	d=z-s[n+1];
	sr=z+d; fr=fff(sr);
	shrink=0;
	if fr<f[1] then // try to extend the simplex
		se=z+2*d; fe=fff(se);
		if fe<fr; s[n+1]=se; else; s[n+1]=sr; endif;
	elseif fr<f[n] then s[n+1]=sr;
	elseif fr<f[n+1] then
		so=z+d/2; fo=fff(so);
		if fo<=f[n+1]; s[n+1]=so; 
		else shrink=1;
		endif;
	else
		si=z-d/2; fi=fff(si);
		if fi<=f[n+1] then s[n+1]=si;
		else shrink=1;
		endif;
	endif;
	if shrink then
		loop 2 to n+1;
			s[#]=(s[1]+s[#])/2;
		end;
	endif;
	return s;
endfunction

function nelderdemo (s,fff)
## Do a demo of the Nelder Mead method in 2 variables
## plotting each step.
## Return the last simplex s
	hold on; 
	c=color(red);
	title("Press scpace or return!");
	h=(s_s[1])'; plot(h[1],h[2]);
	repeat
		if key("Press return")!=32 then 
			hold off; color(c); break; 
		endif;
		s=nelderstep(s,fff);
		h=(s_s[1])'; plot(h[1],h[2]);
	end
	return s;
endfunction

