comment 
Onedimensional Kriging
krigingint(x,y,mu,a=1)
krigingf(t,l,mu,x,a=1)
kriginga (a,x,y)
krigingplot (x,y,a,x1=-1,x2=1)
endcomment

function krigingint (x,y,mu,a=1)
## Interpolate x,y with functions
## f(t)=e^(a*(t-x)^2).
	A=exp(-a*(x-x')^2);
	return A\(y-mu)';
endfunction

function krigingf (t,l,mu,x,a=1)
## Compute the Kriging function
## at points t.
	return mu+l'.exp(-a*(t-x')^2);
endfunction

function kriginga (a,x,y)
## Compute the Kriging maximal error
## for values of a.
	mu=mean(y);
	n=cols(y);
	s=zeros(1,n);
	loop 1 to n;
		x1=x[1:#-1]|x[#+1:n];
		y1=y[1:#-1]|y[#+1:n];
		s[#]=y[#]-krigingf(x[#],krigingint(x1,y1,mu,a),mu,x1,a);
	end;
	return max(abs(s));
endfunction

function krigingplot (x,y,a,x1=-1,x2=1)
## Compute the Kriging interpolation
## and plot the function and the points.
	mu=mean(y);
	l=krigingint(x,y,mu,a);
	p=plot2d("krigingf",x1+0,x2+0;l,mu,x,a);
	plot2d(x,y,points=1,add=1);
	return p;
endfunction;
