// Sets

comment
Functions for sets.
endcomment

// Set functions union, intersection etc. (Version 1.0, Nov 2009 by
// WK)

// Load with "load sets"

function union()
## All different values of the args
	a:=[];
	loop 1 to argn()
		y:=getarg(#);
		s:=rows(y);
		if s>1 then y:=redim(y,1,s*cols(y)); endif
		if !isempty(y) then a:=a|y; endif
	end
	if length(a)<2 then return a; endif
	{a,y}=unique(a);
	return a
endfunction;

// necessary because of the existence of 0xn and nx0 matrices

function isempty (mat:numerical)
## 1 if mat is an empty matrix
	return !prod(size(mat));
endfunction;

function %isequal2 (x1:numerical,x2:numerical)
## 1 if x1 and x2 are equal; used by isequal() and isequalset()
	a=prod(size(x1)); b=prod(size(x2)); // necessary because of 0xn and nx0 matrizes
	if a==0 || b==0 then //all empty matrices are considered to be equal
		if a<>0 || b<>0 then return 0; 
		else return 1; 
		endif; 
	endif  
	if (rows(x1)<>rows(x2))||(cols(x1)<>cols(x2)) then return 0; endif
	return all(x1==x2)
endfunction;

function %intersection2(x1,x2)
## used by intersection() and difference()
	a:=union(x1)|union(x2);
	if length(a)<2 then return []; endif
	y:=sort(a);
	diffy:=differences(y);
	if all(diffy) then return []; endif 
	return y[nonzeros(!(1|diffy))];
endfunction;

function isequal()
## 1 if all args are equal matrices, 
## empty matrices are considered to be equal to each other
	if argn()<2 then error("isequal needs at least 2 arguments"); endif
	loop 2 to argn()
		if %isequal2(arg1,getarg(#)) == 0, then return 0; endif
	end
	return 1;
endfunction;

function isequalset()
## 1 if all args are equal sets
	if argn()<2 then error("isequalset needs at least 2 arguments"); endif
	a:=union(arg1);
	loop 2 to argn()
		if %isequal2(a,union(getarg(#))) == 0, then return 0; endif
	end
	return 1;
endfunction;

function issubset(x:numerical,y:numerical)
## returns 1 if x is a subset of y, 
## bear in mind that an empty set is always a subset of y
	return isempty(difference(x,y));
endfunction;

function intersection()
## intersection(args) returns only the values that exist in every arg.
	if argn()<2 then 
		if argn()==1 then return union(arg1); else return []; endif
	endif
	a:=arg1;
	loop 2 to argn()
		a:=%intersection2(a,getarg(#));
	end
	return a;
endfunction;

function difference (universe)
## Values that exist in universe but not in the other args.
	if isempty(universe) then return []; endif
	if argn()==1 then return union(universe); endif
	a:=sort(union(args(2))|union(universe));
	if length(a)<2 then return a; endif
	diffy:=!differences(a);
	diffy:=(0|diffy) + (diffy|0);
	return %intersection2(a[nonzeros(!diffy)],universe);
endfunction;

