comment
Shows vectorfields and solutions of DGL.
E.g.
dgltest("x*sin(y)",-pi,pi,-pi,pi);
dgl2test("y","-x",-1,1,-1,1);
endcomment

function dgltest (expr,x1,x2,y1,y2,nx=20,ny=20,n=10)
## Draw a vector field and let the user click to a starting point.
## expr must be an expression in x and y.
## x1,x2,y1,y2 are the rectangle, which will be drawn.
## nx,ny is the number of discretization points for the drawing.
## n is the number of discretization (times nx) for the DGL.
	vectorfield(expr,x1,x2,y1,y2,nx,ny);
	title("Click for a solution (>END<)");
	repeat
		p=mouse();
		if (p[2]>y2); break; endif;
		hold on;
		linewidth(2); 
		t=p[1]:(x2-x1)/(nx*n):x2;
		plot(t,heun(expr,t,p[2]));
		t=p[1]:(x1-x2)/(nx*n):x1;
		plot(t,heun(expr,t,p[2]));
		linewidth(1);
		hold off;
	end;
	return plot();
endfunction

function fff2test (X,Y,expr1,expr2)
	y1=expreval(expr1,Y[1],y=Y[2]);
	y2=expreval(expr2,Y[1],y=Y[2]);
	return [y1,y2];
endfunction

function dgl2test (expr1,expr2,x1,x2,y1,y2,nx=20,ny=20,t=0:0.01:5)
## Draw a vector field and let the user click to a starting point.
## x1,x2,y1,y2 are the rectangle, which will be drawn.
## expr1 is the x-value of the derivative and expr2 its y-value.
## Both expressions must be expressions in x and y.
## E.g., v'=f(v), with f([x,y])=[expr1,expr2].
## nx,ny is the number of discretization points for the drawing.
## t determines, where the ODE will be evaluated.
	vectorfield2(expr1,expr2,x1,x2,y1,y2,nx,ny);
	title("Click for a solution (>END<)");
	repeat
		p=mouse();
		if (p[2]>y2); break; endif;
		hold on;
		linewidth(2);
		y=heun("fff2test",t,p;expr1,expr2);
		plot(y[1],y[2]);
		linewidth(1);
		hold off;
	end;
	return plot();
endfunction


