#include "dlldef.h"
#include <stdio.h>

// List stuff (lists are growing arrays)

typedef struct
{
	int capacity,n; // n=current size
	void **p;
} 
list;

list *createlist (int capacity)
// create a new empty list with initial capacity
{
	list *v=(list *)malloc(sizeof(list));
	v->capacity=capacity; v->n=0;
	v->p=(void **)malloc(capacity*sizeof(void *));
	return v;
}

void *getelement (list *v, int i)
// return element i
{
	if (i<0 || i>=v->n) return 0;
	return v->p[i];
}

void addelement (list *v, void *x)
// add new element
{
	if (v->n+1 >= v->capacity) // full?
	{
		// print("Extend capacity %d\n",v->capacity);
		void **pnew=(void **)malloc(2*v->capacity*sizeof(void *));
		int i;
		for (i=0; i<v->n; i++) pnew[i]=v->p[i];
		free(v->p);
		v->capacity=2*v->capacity;
		v->p=pnew;
	}
	v->p[v->n]=x;
	v->n++;
}

void *removeelement (list *v, int i)
// remove element i, return element
{
	if (i<0 || i>=v->capacity) return 0;
	void *p=v->p[i];
	int j;
	for (j=i; j<v->capacity; j++)
		v->p[j]=v->p[j+1];
	v->n--;
	return p;
}

// List of named lists (lists with name to identify)

typedef struct 
{
	char *name;
	list *v;
} 
namedlist;

namedlist *makenamedlist (char *name)
// new named list with this name
{
	namedlist *nv=(namedlist *)malloc(sizeof(namedlist));
	nv->name=(char *)malloc(strlen(name)+1);
	strcpy(nv->name,name);
	nv->v=createlist(32);
	return nv;
}

list *findnamedlist (list *w, char *name)
// find the list with that name in w
{
	if (w==0) return 0;
	int i;
	for (i=0; i<w->n; i++)
	{
		namedlist *nv=(namedlist *)(w->p[i]);
		if (strcmp(name,nv->name)==0) return nv->v;
	}
	return 0;
}

// Hashtable stuff

list **makehashtable ()
{
	list **v=(list **)malloc(256*sizeof(list *));
	int i;
	for (i=0; i<256; i++) v[i]=0;
	return v;
}

#define BYTE unsigned char

BYTE hash (char *s)
{
	BYTE res=0;
	while (*s)
	{
		BYTE x=(BYTE)*s;
		res=(BYTE)((res*7)+x);
		s++;
	}
	return res;
}

// Global hashtable of lists

list **L=0;

list *find (list **lists, char *name)
{
	BYTE b=hash(name);
	list *nv=lists[(int)b];
	if (!nv)
	{
		nv=createlist(8);
		lists[(int)b]=nv;
	}
	return nv;
}

// Global hashtable for variables

list **G=0;

typedef struct 
{
	char *name;
	header *hd;
} 
variable;

variable *makevariable (char *name, header *h)
{
	variable *v=(variable *)malloc(sizeof(variable));
	v->name=(char *)malloc(sizeof(name)+1);
	strcpy(v->name,name);
	v->hd=h;
	return v;
}

variable *findvariable (list *w, char *name)
// find the list with that name in w
{
	if (w==0) return 0;
	int i;
	for (i=0; i<w->n; i++)
	{
		variable *nv=(variable *)(w->p[i]);
		if (strcmp(name,nv->name)==0) return nv;
	}
	return 0;
}

// Utilities for Euler headers

char *getname (header *hd)
// extract the name from the header, or set error
{
	if (hd->type==s_string) return stringof(hd);
	error=1; return 0;
}

// Exported functions

EXPORT char * makelist (header *hd[], int np, char *ramstart, char *ramend)
// make a new list with a name, makelist(name)
{
	start(ramstart,ramend); // required !!!
	
	if (L==0) L=makehashtable();	
	
	char *name=getname(hd[0]);
	CHECK(name,"Need a string for makelist");
	
	list *v=findnamedlist(find(L,name),name);
	CHECK(!v,"List does already exist. Remove list first.");

	namedlist *nv=makenamedlist(name);
	addelement(find(L,name),nv);
	
	return 0; // no result
}

EXPORT char * listadd (header *hd[], int np, char *ramstart, char *ramend)
// add an Euler element to a list, listadd(name,element)
{
	start(ramstart,ramend); // required !!!
	
	char *name=getname(hd[0]);
	CHECK(name,"Need a string and an object for listadd");
	
	list *v=findnamedlist(find(L,name),name);
	CHECK(v,"List not found");
	
	header *h=(header *)malloc(hd[1]->size);
	memcpy(h,hd[1],hd[1]->size);
	addelement(v,h);
	
	return 0; // no result
}


EXPORT char * listsize (header *hd[], int np, char *ramstart, char *ramend)
// get the size of a list, listsize(name)
{
	start(ramstart,ramend); // required !!!
	
	char *name=getname(hd[0]);
	CHECK(name,"Need a string and an object for listadd");
	
	list *v=findnamedlist(find(L,name),name);
	CHECK(v,"List not found");
	
	new_real(v->n);	
	
	return newram;
}


EXPORT char * listget (header *hd[], int np, char *ramstart, char *ramend)
// get an Euler element from a list, listget(name,i)
{
	start(ramstart,ramend); // required !!!
	
	char *name=getname(hd[0]);
	CHECK(name,"Need a string and an object for listadd");
	
	list *v=findnamedlist(find(L,name),name);
	CHECK(v,"List not found");	
	
	int n=getint(hd[1]);
	IFERROR("Need an integer for listget.");
	header *h=(header *)getelement(v,n-1);
	CHECK(h,"Index out of range for list.");
	memcpy(newram,h,h->size);
	newram+=h->size;
	
	return newram;
}


EXPORT char * listremove (header *hd[], int np, char *ramstart, char *ramend)
// remove an element from a list, listremove(name,i)
{
	start(ramstart,ramend); // required !!!
	
	char *name=getname(hd[0]);
	CHECK(name,"Need a string and an object for listadd");
	
	list *v=findnamedlist(find(L,name),name);
	CHECK(v,"List not found");		
	
	int n=getint(hd[1]);
	IFERROR("Need an integer for listget.");
	header *h=(header *)getelement(v,n-1);
	CHECK(h,"Index out of range for list.");
	memcpy(newram,h,h->size);
	newram+=h->size;
	
	free(removeelement(v,n-1));
	
	return newram;
}

EXPORT char * removelist (header *hd[], int np, char *ramstart, char *ramend)
// remove a list, removelist(name)
{
	start(ramstart,ramend); // required !!!
	
	char *name=getname(hd[0]);
	CHECK(name,"Need a string and an object for listadd");

	int i,j;
	list *V=find(L,name);
	for (i=0; i<V->n; i++)
	{
		namedlist *nv=(namedlist *)(V->p[i]);
		if (strcmp(name,nv->name)==0)
		{
			for (j=0; j<nv->v->n; j++) free(nv->v->p[j]);
			free(nv->v);
			free(removeelement(V,i));
			return 0;
		}
	}	
	
	return 0;
}

EXPORT char * removeall (header *hd[], int np, char *ramstart, char *ramend)
// remove all lists, removeall()
{
	int i,j,k;
	
	if (!L) return 0;
	
	for (k=0; k<256; k++)
	{
		list *V=L[k];
		if (V)
		{
			for (i=0; i<V->n; i++)
			{
				namedlist *nv=(namedlist *)(V->p[i]);
				for (j=0; j<nv->v->n; j++) free(nv->v->p[j]);
				free(nv->v);
				free(nv);
			}
			free(L[k]);
		}
		L[k]=0;
	}
		
	return 0;
}

EXPORT char * globalset (header *hd[], int np, char *ramstart, char *ramend)
{
	start(ramstart,ramend); // required !!!
	
	if (!G) G=makehashtable();
	
	char *name=getname(hd[0]);
	CHECK(name,"Need a string and an object for globalset");
	
	list *w=find(G,name);
	variable *v=findvariable(w,name);
	
	header *h=(header *)malloc(hd[1]->size);
	memcpy(h,hd[1],hd[1]->size);

	if (!v)
	{
		v=makevariable(name,h);
		addelement(w,v);
	}
	else
	{
		free(v->hd);
		v->hd=h;
	}
	
	return 0; // no result
}

EXPORT char * globalget (header *hd[], int np, char *ramstart, char *ramend)
{
	start(ramstart,ramend); // required !!!
	
	CHECK(G,"No global variables defined.\n");
	
	char *name=getname(hd[0]);
	CHECK(name,"Need a string for globalget");
	
	list *w=find(G,name);
	variable *v=findvariable(w,name);
	if (!v)
	{
		sprintf(ram,"Global variable %s not found.\n",name);
		return ram;
	}

	memmove(ram,(char *)v->hd,v->hd->size);
	newram+=v->hd->size;
	
	return newram;
}

EXPORT char * listglobals (header *hd[], int np, char *ramstart, char *ramend)
{
	start(ramstart,ramend);
	
	header *result=new_smatrix();
	IFERROR("Out of RAM");
	
	if (G)
	{
		int i,k;
		for (k=0; k<256; k++)
		{
			list *w=G[k];
			if (w)
			{
				for (i=0; i<w->n; i++)
				{
					variable *v=(variable *)w->p[i];
					appendstring(result,v->name);
					IFERROR("Out of RAM");
				}
			}
		}
	}
	
	return newram;
}

EXPORT char * listlists (header *hd[], int np, char *ramstart, char *ramend)
{
	start(ramstart,ramend);
	header *result=new_smatrix();

	if (L)
	{
		int i,j,k;
		for (k=0; k<256; k++)
		{
			list *V=L[k];
			if (V)
			{
				for (i=0; i<V->n; i++)
				{
					namedlist *nv=(namedlist *)(V->p[i]);
					appendstring(result,nv->name);
					IFERROR("Out of RAM");
				}
			}
		}
	}
	
	return newram;
}

EXPORT clear ()
{
	if (L)
	{
		int i,j,k;
		for (k=0; k<256; k++)
		{
			list *V=L[k];
			if (V)
			{
				for (i=0; i<V->n; i++)
				{
					namedlist *nv=(namedlist *)(V->p[i]);
					for (j=0; j<nv->v->n; j++) free(nv->v->p[j]);
					free(nv->v);
					free(nv);
				}
				free(L[k]);
				L[k]=0;
			}
		}
		free(L);
		L=0;	
	}
	
	if (G)
	{
		int i,j,k;
		for (k=0; k<256; k++)
		{
			list *w=G[k];
			if (w)
			{
				for (i=0; i<w->n; i++)
				{
					variable *v=(variable *)w->p[j];
					free(v->name);
					free(v->hd);
					free(v);
				}
				free(w);
			}
		}
		free(G);
		G=0;	
	}
}
