// Lists Library

comment
This class uses a dynamic library to implement global variables and
lists in Euler. 
endcomment

// Euler functions can access global variables, if useglobal or
// global is enabled. But they cannot change the type of global
// variables or their size. The lists library provided a way to set
// global variables.

// Sometimes list of objects with different types are needed. The list
// library provides those lists. The lists have names, and are
// accessed by names. There are functions to add an Euler object to a
// list, to get the item by number, or remove an item from the list.
// To clean, up you can remove a list, or clear all lists.

// The lists are stored in arrays, which can grow if needed. The
// initial capacity is 32 and grows by doubling. The names of the
// lists are stored in a hashtable of capacity 256. This should work
// smooth enough for a few thousands of lists.

// Needs to be loaded with "load lists".

dll("lists","globalset",2);
dll("lists","globalget",1);

function comment globalset (name:string, object)
## Set the a global library variable.
endfunction

function comment globalget (name:string)
## Get a global library variable.
endfunction

dll("lists","makelist",1);
dll("lists","listadd",2);
dll("lists","listsize",1);
dll("lists","listget",2);
dll("lists","listremove",2);
dll("lists","removelist",1);
dll("lists","removeall",0);
dll("lists","listglobals",0);
dll("lists","listlists",0);

function comment makelist (name:string)
## Creates a new list with this name or resets the list.
## If the list exists, it has to be removed first.
endfunction

function comment listadd (name:string, object)
## Adds an object to the list.
endfunction

function comment listsize (name:string)
## The length of the list
endfunction

function comment listget (name:string, i:index)
## Gets object number i from the list.
endfunction

function comment listremove (name:string, i:index)
## Removes object number i from the list.
endfunction

function comment removelist (name:string)
## Removes a list
endfunction

function comment removeall ()
## Remove all lists and release the memory.
endfunction

function comment listglobals ()
## List all global variables in the library (unsorted).
endfunction

function comment listlists ()
## List all lists in the library (unsorted).
endfunction

submenu "Global Library"
addmenu globalset(?name,?value)
addmenu globalget(?name)
addmenu makelist(?name)
addmenu listadd(?name,?value)
addmenu listget(?name,?index)
addmenu listsize(?name)
addmenu listremove(?name,?index)
addmenu removelist(?name)
