// Logarithmic Plots

comment
Logarithmic plots.
endcomment

// These function should not be called diretly. Use plot2d to produce log plots.

function xlogplot (x=1, y=0, xbase=10, color=1, points=false, frame=1)
## Draws y(x) in a semilog paper with base xbase.
##
## See: plot2d
	global logredraw;
	if !holding();clg;endif;
	if iscomplex(x); xe=abs(x); else xe=x; endif;
	if iscomplex(y); ye=abs(y); else ye=y; endif;
	xl=logbase(xe,xbase);
	
	global defaultgridcolor;
	c=color(defaultgridcolor);
	
	if holding() && !logredraw
		hd=holding();
		p=plot();
	else
		expi=floor(logbase(totalmin(xe),xbase));
		expf=ceil(logbase(totalmax(xe),xbase));

		if isNAN(expi) or isNAN(expf) then
			error("Error in logarithmic range.");
		endif;

		xt=xlogticks(expi,expf,xbase);
		if rows(ye)==1
			yt=ticks(min(ye),max(ye));
		elseif rows(ye)>1
			yt=ticks(totalmin(ye)),totalmax(ye));
		endif;

		p=setplot([expi,expf,totalmin(ye),totalmax(ye)]);
		
		h=textheight(); w=textwidth();
		
		hd=holding(1);
		global defaultgrid2;
		ls=linestyle(defaultgrid2);
		lw=linewidth(1);
		
		pf=fullplotarea();		
		
		for i=1 to length(yt);
			plot([expi,expf],[yt[i],yt[i]]);
			d=toscreen([pf[1],yt[i]]);
			rtext(printf("%g",yt[i]),[d[1]-textwidth(),d[2]-h/2]);
		end;

		for i=1 to length(xt);
			plot([xt[i],xt[i]],[p[3],p[4]]);
		end;

		for k=1 to length(xt);
			i=floor(xt[k]+0.2);
			d=toscreen([i,pf[3]]);
			rtext(printf("%g",xbase),[d[1]+w,d[2]+0.75*h]);
			text(printf("%g",i),[d[1]+w,d[2]+h/4]);
		end;

		linewidth(lw);
		linestyle(ls);
		
		if frame then frame(); endif;
	endif;
	
	_color(color);
	if points then mark(xl,ye);
	else plot(xl,ye);
	endif;
	holding(hd);
	logredraw=0;
	color(c);
	return p;
endfunction;

function ylogplot (x, y, ybase=10, color=1, points=false, frame=1)
## Draws y(x) in a semilog paper with base ybase.
##
## See: plot2d
	global logredraw;
	
	if !holding() then clg; endif;
	if iscomplex(x) then xe=abs(x); else xe=x; endif;
	if iscomplex(y) then ye=abs(y); else ye=y; endif;
	yl=logbase(ye,ybase);
		
	global defaultgridcolor;
	c=color(defaultgridcolor);
	
	if holding && !logredraw then
		hd=holding;
		p=plot();
	else
		expi=floor(logbase(totalmin(ye),ybase));
		expf=ceil(logbase(totalmax(ye),ybase));
		
		if isNAN(expi) or isNAN(expf) then
			error("Error in logarithmic range.");
		endif;

		xt=ticks(totalmin(xe),totalmax(xe));
		yt=xlogticks(expi,expf,ybase);

		p=setplot([totalmin(xe),totalmax(xe),expi,expf]);
		pf=fullplotarea();

		h=textheight(); w=textwidth();

		hd=holding(1);
		global defaultgrid2;
		ls=linestyle(defaultgrid2);
		lw=linewidth(1);
		
		for i=1 to length(yt);
			plot([p[1],p[2]],[yt[i],yt[i]]);
		end;
		
		for k=1 to length(yt);
			i=floor(yt[k]+0.2);
			d=toscreen([pf[1],i]);
			rtext(printf("%g",ybase),[d[1]-40,d[2]-h/2]);
			text(printf("%g",i),[d[1]-40,d[2]-h]);
		end;

		for i=1 to length(xt);
			plot([xt[i],xt[i]],[p[3],p[4]]);
			d=toscreen([xt[i],pf[3]]);
			ctext(printf("%g",xt[i]),[d[1],d[2]+h/2]);
		end;

		linewidth(lw);
		linestyle(ls);
		if frame then frame(); endif;
	endif
	
	_color(color);
	
	if points then mark(xe,yl);
	else plot(xe,yl);
	endif;
	
	color(c);
	
	holding(hd);
	logredraw=0;
	return p;
endfunction;

function xylogplot(x, y, xbase=10, ybase=10, color=1, points=0, frame=1)
## Draws y(x) in a loglog paper with base xbase and ybase
##
## See: plot2d
	global logredraw;
	if !holding() then clg; endif;
	if iscomplex(x) then xe=abs(x); else xe=x; endif;
	if iscomplex(y) then ye=abs(y); else ye=y; endif;
	
	xl=logbase(xe,xbase);
	yl=logbase(ye,ybase);
	
	global defaultgridcolor;
	c=_color(defaultgridcolor);
	
	if holding() && !logredraw
		hd=holding();
		p=plot();
	else
		expix=floor(logbase(totalmin(xe),xbase));
		expfx=ceil(logbase(totalmax(xe),xbase));
		expiy=floor(logbase(totalmin(ye),ybase));
		expfy=ceil(logbase(totalmax(ye),ybase));
		
		if isNAN(expix) or isNAN(expfx) or isNAN(expiy) or isNAN(expfy) then
			error("Error in logarithmic range.");
		endif;
		
		xt=xlogticks(expix,expfx,xbase);
		yt=xlogticks(expiy,expfy,ybase);

		p=setplot([expix,expfx,expiy,expfy]);
		pf=fullplotarea();

		h=textheight(); w=textwidth();

		hd=holding(1);
		global defaultgrid2;
		ls=linestyle(defaultgrid2);
		lw=linewidth(1);

		for i=1 to length(xt);
			plot([xt[i],xt[i]],[p[3],p[4]]);
		end;
		
		for k=1 to length(xt);
			i=floor(xt[k]+0.2);
			d=toscreen([i,pf[3]]);
			rtext(printf("%g",xbase),[d[1]+w,d[2]+0.75*h]);
			text(printf("%g",i),[d[1]+w,d[2]+h/4]);
		end;

		for i=1 to length(yt);
			plot([p[1],p[2]],[yt[i],yt[i]]);
		end;
		
		for k=1 to length(yt);
			i=floor(yt[k]+0.2);
			d=toscreen([pf[1],i]);
			rtext(printf("%g",ybase),[d[1]-40,d[2]-h/2]);
			text(printf("%g",i),[d[1]-40,d[2]-h]);
		end;

		linewidth(lw);
		linestyle(ls);
		if frame then frame(); endif;
	endif;
	_color(color);
	if points then mark(xl,yl);
	else plot(xl,yl);
	endif;
	_color(c);
	holding(hd);
	logredraw=0;
	return p;
endfunction;

function logspace (a,b,n=50,base=10)
## logspace(a,b,n) generates n log spaced points in [a,b]
##
## base specifies the based log used.
	if a>b
		tmp=a;
		a=b;
		b=tmp;
	endif;
	return base^(linspace(logbase(a,base),logbase(b,base),n-1));
endfunction

function xticks (aa=0,bb=0)
	if argn==2; a=aa; b=bb;
	elseif argn==1; a=min(aa); b=max(aa);
	else error("Wrong arguments for ticks");
	endif;
	if (b>1e30); b=1e30; endif;
	if (a<-1e30); a=-1e30; endif;
	if (a>=b); b=a+1; endif;
	tick=10^floor(log(b-a)/log(10)-0.4);
	if b-a>10*tick then tick=tick*2; endif;
	if b-a<5*tick then tick=tick/2; endif;
	if tick<1 then tick=1; endif;
	res=(floor(a/tick))*tick:tick:(ceil(b/tick))*tick;
	return res[nonzeros(res>=aa && res<=bb)];
endfunction;

function xlogticks (expi,expf,base)
	return xticks(expi,expf);
endfunction;

